/*  Thyme can send your location via SMS.
    Copyright (C) 2011,2012 Josef Hahn <thyme@ginger.ws>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package ws.ginger.thyme;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.content.res.Resources;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.provider.ContactsContract.CommonDataKinds.Email;
import android.provider.ContactsContract.CommonDataKinds.Phone;
import android.provider.ContactsContract.Contacts;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.TextView;
import android.widget.Toast;

public class ThymeActivity extends Activity {
		
	// internal key for our preferences storage place within Android
	public static final String PREFS_NAME = "ThymePrefs";
	
	// we access the preferences from this object
	private SharedPreferences settings;
	
	@Override
	protected void onResume() {
		super.onResume();
	    boolean firsttime = settings.getBoolean("firsttime", true);
	    Editor e = settings.edit();
	    e.putBoolean("firsttime", false);
	    e.commit();
	    if (firsttime) {
	    	AlertDialog ad = new AlertDialog.Builder(this).create();
	    	ad.setCancelable(false);
	    	ad.setMessage("This program sends SMS which potentially costs money. It's not well tested yet. You use it at your own risk!");
	    	ad.setButton("OK", new DialogInterface.OnClickListener() {
	    	    public void onClick(DialogInterface dialog, int which) {
	    			SharedPreferences settings = getSharedPreferences(PREFS_NAME, 0);
	    		    Editor e = settings.edit();
	    		    e.putBoolean("firsttime", false);
	    		    e.commit();
	    	        dialog.dismiss();                    
	    	    }
	    	});
	    	ad.show();
	    	
	    }
	}

	private Button btnSend;
	private Button btnContacts;
	private CheckBox chkLatlon;
	private CheckBox chkMaplink;
	private CheckBox chkGpsNeeded;
	private TextView editPhoneno;
	private TextView editCustomtxt;

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        btnSend = (Button)findViewById(R.id.button2);
        btnContacts = (Button)findViewById(R.id.button1);
        chkLatlon = (CheckBox)findViewById(R.id.checkBox1);
        chkMaplink = (CheckBox)findViewById(R.id.checkBox2);
        chkGpsNeeded = (CheckBox)findViewById(R.id.checkBox3);
        editPhoneno = (TextView)findViewById(R.id.editText1);
        editCustomtxt = (TextView)findViewById(R.id.editText3);
    
		settings = getSharedPreferences(PREFS_NAME, 0);
		        
        if (savedInstanceState!=null){
	        chkLatlon.setChecked(savedInstanceState.getBoolean("latlon", true));
	        chkMaplink.setChecked(savedInstanceState.getBoolean("maplink", true));
	        chkGpsNeeded.setChecked(savedInstanceState.getBoolean("gpsneeded", true));
	        editPhoneno.setText(savedInstanceState.getString("phoneno"));
	        editCustomtxt.setText(savedInstanceState.getString("customtxt"));
        } else {
        	chkLatlon.setChecked(settings.getBoolean("chk_latlon", true));
	        chkMaplink.setChecked(settings.getBoolean("chk_maplink", true));
	        chkGpsNeeded.setChecked(settings.getBoolean("chk_gps", true));
        }
        
        btnSend.setOnClickListener(btnSendListener);
        btnContacts.setOnClickListener(btnContactsListener);
        
        chkLatlon.setOnCheckedChangeListener(new OnCheckedChangeListener() {
			
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
			    storePref("chk_latlon", chkLatlon.isChecked());
			}
			
		});
        
        chkMaplink.setOnCheckedChangeListener(new OnCheckedChangeListener() {
			
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
			    storePref("chk_maplink", chkMaplink.isChecked());
			}
			
		});
        
        
        chkGpsNeeded.setOnCheckedChangeListener(new OnCheckedChangeListener() {
			
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
			    storePref("chk_gps", chkGpsNeeded.isChecked());
			}
			
		});
        
    }
    
    // small helper storing preferences into our store
    private void storePref(String key, boolean value) {
	    Editor e = settings.edit();
	    e.putBoolean(key, value);
	    e.commit();
    }
    
    
	@Override
	protected void onSaveInstanceState(Bundle outState) {
		outState.putBoolean("latlon", chkLatlon.isChecked());
		outState.putBoolean("maplink", chkMaplink.isChecked());
		outState.putBoolean("gpsneeded", chkGpsNeeded.isChecked());
		outState.putString("phoneno", editPhoneno.getText().toString());
		outState.putString("customtxt", editCustomtxt.getText().toString());
		super.onSaveInstanceState(outState);
	}


	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (resultCode == RESULT_OK) {
            switch (requestCode) {
            case CONTACT_PICKER_RESULT:
            	Uri uri = data.getData();
            	Cursor cursor = getContentResolver().query(uri,
            			new String[]{Contacts._ID}, 
            			null, null, null);
            	if (cursor.moveToFirst()) {
            		long contactId = cursor.getLong(0);
                    Cursor cursor2 = getContentResolver().query(		
                			Phone.CONTENT_URI,
                            new String[]{Phone.NUMBER, Phone.TYPE},
                            Phone.CONTACT_ID + "=" + contactId, null, Phone.IS_SUPER_PRIMARY + " DESC");
                    String mobileNr="";
                    String phoneNr="";
                	while (cursor2.moveToNext()) {
                		phoneNr=cursor2.getString(0);
                		if (cursor2.getInt(1)==Phone.TYPE_MOBILE) {
                			mobileNr=cursor2.getString(0);
                		}
                    }
                	editPhoneno.setText( mobileNr.length()>0?mobileNr:phoneNr  );
                	cursor2.close();
                    break;
                }
            	cursor.close();
            }
        }
    }
 
    private static final int CONTACT_PICKER_RESULT = 1001;
    
    private int getCustomMsgMaxlen() 
    {
		int l = 160;
		if (chkLatlon.isChecked()) l-=22;
		if (chkMaplink.isChecked()) l-=64;
		return l;
    }
    

    
    private OnClickListener btnContactsListener = new OnClickListener()
    {
        public void onClick(View v)
        {                  
            Intent contactPickerIntent = new Intent(Intent.ACTION_PICK,
                    Contacts.CONTENT_URI);
            startActivityForResult(contactPickerIntent, CONTACT_PICKER_RESULT);
        }
    };
    
    private OnClickListener btnSendListener = new OnClickListener()
    {
        public void onClick(View v)
        {
        	int customMsgMaxLen = getCustomMsgMaxlen();
        	if (editCustomtxt.length() <= customMsgMaxLen) {
	            Intent sendIntent=new Intent("SEND_MESSAGE", null, ThymeActivity.this, ThymeService.class);
	            sendIntent.putExtra("phoneno", editPhoneno.getText().toString());
	            sendIntent.putExtra("custommsg", editCustomtxt.getText().toString());
	            sendIntent.putExtra("gpscoords", chkLatlon.isChecked());
	            sendIntent.putExtra("maplink", chkMaplink.isChecked());
	            sendIntent.putExtra("gpsneeded", chkGpsNeeded.isChecked());
	            startService(sendIntent);
	            finish();
        	} else {
        		Context context = getApplicationContext();
        		CharSequence text = "Your message is "+ (editCustomtxt.length() - customMsgMaxLen) + " characters too long." ;
        		int duration = Toast.LENGTH_SHORT;
        		Toast toast = Toast.makeText(context, text, duration);
        		toast.show();
        	}
        }
    };
    
    
}