/*  Thyme can send your location via SMS.
    Copyright (C) 2011,2012 Josef Hahn <thyme@ginger.ws>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package ws.ginger.thyme;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import android.app.Activity;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.IBinder;
import android.telephony.SmsManager;
import android.widget.Toast;

public class ThymeService extends Service {
	
	private boolean smsSending;

    private int accuracyCounter;
	
	private BroadcastReceiver smsReceivedReceiver = new BroadcastReceiver(){
		@Override
		public void onReceive(Context arg0, Intent arg1) {
			switch (getResultCode())
			{
			case Activity.RESULT_OK:
				notification("SMS sent", "SMS sent to '"+req.phoneno+"'");
				break;
			case SmsManager.RESULT_ERROR_GENERIC_FAILURE:
				notification("SMS NOT sent", "Generic SMS service failure");
				break;
			case SmsManager.RESULT_ERROR_NO_SERVICE:
				notification("SMS NOT sent", "No service");
				break;
			case SmsManager.RESULT_ERROR_NULL_PDU:
				notification("SMS NOT sent", "NULL PDU");
				break;
			case SmsManager.RESULT_ERROR_RADIO_OFF:
				notification("SMS NOT sent", "Radio is turned off.");
				break;
			}
			req=null;
			smsSending=false;
			removeOngoing();
			stopSelf();
		}
	};
	
	
	LocationListener locationListener = new LocationListener() {
		
		public void onLocationChanged(Location location) {
			if (smsSending || req==null) return;

            accuracyCounter--;
            if (accuracyCounter > 0)
                return;

            if (new Date().getTime() - location.getTime() > 10000)
                return;

            ongoing("Sending SMS", "Sending location information...");

			SmsManager smsman=SmsManager.getDefault();
			StringBuilder txt=new StringBuilder();
			
			if (req.custommsg.length()>0) {
				txt.append(req.custommsg);
			} else {
				txt.append("Here I am");
			}
			
			double lon=location.getLongitude();
			double lat=location.getLatitude();

			
			locationManager.removeUpdates(locationListener);
			
			if (req.gpscoords) {
				txt.append( String.format(" - %.4f%s %.4f%s",
						Math.abs(lat),
						(lat>0?"N":"S"),
						Math.abs(lon),
						(lon>0?"E":"W")
						));
			}
			if (req.maplink) {
				String link=String.format(" - http://osm.org/?mlon=%.4f&mlat=%.4f&zoom=15",lon,lat);
				txt.append(link);
			}
			PendingIntent pi = PendingIntent.getBroadcast(ThymeService.this, 0, new Intent("SENT"), 0);

			smsman.sendTextMessage(req.phoneno, null, txt.toString(), pi, null);
			smsSending=true;
	    }

	    public void onProviderEnabled(String provider) {}

	    public void onProviderDisabled(String provider) {
	    	removeOngoing();
	    	notification("Unable to get location", "Enable the location source in system settings & try again.");
	    }

		@Override
		public void onStatusChanged(String provider, int status, Bundle extras) {
			
		}
	};

	LocationManager locationManager;
	
	@Override
	public void onCreate() {
		super.onCreate();
		registerReceiver(smsReceivedReceiver, new IntentFilter("SENT"));
	}
	
	@Override
	public void onDestroy() {
		unregisterReceiver(smsReceivedReceiver);
		super.onDestroy();
	}

	private ThymeRequest req = null;
	
	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		if (req!=null) {
			notification("Could not send", "Couldn't send due to a pending request.");
		} else {
			req = new ThymeRequest();
			req.phoneno=intent.getExtras().getString("phoneno");
			req.custommsg=intent.getExtras().getString("custommsg");
			req.gpscoords = intent.getExtras().getBoolean("gpscoords");
			req.maplink = intent.getExtras().getBoolean("maplink");
			req.gpsneeded=intent.getExtras().getBoolean("gpsneeded");
			req.startId=startId;
			smsSending=false;
            accuracyCounter = 5;
			locationManager = (LocationManager) this.getSystemService(Context.LOCATION_SERVICE);
			locationManager.requestLocationUpdates( req.gpsneeded?LocationManager.GPS_PROVIDER:LocationManager.NETWORK_PROVIDER , 1000, 0, locationListener);
			ongoing("Waiting for location", "Waiting for the "+(req.gpsneeded?"GPS":"GSM-based")+" location");
		}
		return Service.START_NOT_STICKY;
	}
	
	private void notification(String shortmsg, String msg) {
		long when = System.currentTimeMillis();         // notification time
		Context context = getApplicationContext();      // application Context
		Intent notificationIntent = new Intent(this, ThymeActivity.class);
		PendingIntent contentIntent = PendingIntent.getActivity(this, 0, notificationIntent, 0);
		Notification notification = new Notification(R.drawable.thyme, shortmsg, when);
		notification.setLatestEventInfo(context, "Thyme", msg, contentIntent);
		int notifyId=(int)( Math.random() * Integer.MAX_VALUE-2 )+2;
		String ns = Context.NOTIFICATION_SERVICE;
		NotificationManager mNotificationManager = (NotificationManager) getSystemService(ns);
		mNotificationManager.notify(notifyId, notification);
	}
	
	private void removeOngoing() {
		String ns = Context.NOTIFICATION_SERVICE;
		NotificationManager mNotificationManager = (NotificationManager) getSystemService(ns);
		mNotificationManager.cancel(1);
	}
	
	private void ongoing(String shortmsg, String msg) {
		long when = System.currentTimeMillis();         // notification time
		Context context = getApplicationContext();      // application Context
		Intent notificationIntent = new Intent(this, ThymeActivity.class);
		PendingIntent contentIntent = PendingIntent.getActivity(this, 0, notificationIntent, 0);
		Notification notification = new Notification(R.drawable.thyme, shortmsg, when);
		notification.setLatestEventInfo(context, "Thyme", msg, contentIntent);
		notification.flags |= Notification.FLAG_ONGOING_EVENT;
		String ns = Context.NOTIFICATION_SERVICE;
		NotificationManager mNotificationManager = (NotificationManager) getSystemService(ns);
		mNotificationManager.notify(1, notification);
	}
	
	@Override
	public IBinder onBind(Intent intent) {
		return null;
	}

}
